<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Str;
use Intervention\Image\Facades\Image as ImageIntervention;
use stdClass;


class ImageController extends Controller
{

    protected $sizes =
        [
            array('width' => 75, 'height' => 75),
            

            array('width' => 150, 'height' => 150),

            array('width' => 200, 'height' => 200),

            array('width' => 300, 'height' => 300),
            
            array('width' => 600, 'height' => 600),
            
            array('width' => 113, 'height' => 113),
            
            array('width' => 250, 'height' => 250),
            
            array('width' => 450, 'height' => 450),

            array('width' => 115, 'height' => 115),

        ];
    protected $available_sizes =
        [
            array('width' => 75, 'height' => 75),
            array('width' => 80, 'height' => 80),

            array('width' => 150, 'height' => 150),

            array('width' => 200, 'height' => 200),

            array('width' => 300, 'height' => 300),

            array('width' => 400, 'height' => 200),
            
            array('width' => 400, 'height' => 400),
            
            array('width' => 600, 'height' => 600),
            
            array('width' => 113, 'height' => 113),
            
            array('width' => 250, 'height' => 250),
            
            array('width' => 450, 'height' => 450),

            array('width' => 115, 'height' => 115),


        ];

    public function handleUpload(Request $request)
    {

        $this->validate($request, [

            'image' => 'image|required|mimes:jpeg,png,jpg,gif,svg|max:2000',
            'seller_id' => 'required|numeric',

        ]);
        $seller_id = $request->seller_id;

        //To be changed if the request will include the related product name

        $product_name = $request->product_name ? Str::slug($request->product_name) : 'product';

        $file = $request->file('image');

        $fileName = $product_name . '_' . substr(Str::uuid()->toString(), 0, -13);

        $path = 'originals/' . $seller_id . '/';

        File::exists($path) ?:
            File::makeDirectory($path, $mode = 0777, TRUE, TRUE);


        if (strtolower($file->getClientOriginalExtension()) != 'jpg') {

            $img = ImageIntervention::make($file);

            $img->encode('jpg')->save($path . $fileName . '.jpg');

        } else
            $file->move($path, $fileName . '.' . $file->getClientOriginalExtension());


        $size = new stdClass();

        $size->width = $request->query->get('width');

        $size->height = $request->query->get('height');

        $size->width ? $this->resizeImageCustomSize($size, $path, $seller_id, $fileName, 'jpg') :
            $this->resizeImage($path, $seller_id, $fileName, 'jpg');

        return response()->json(['url' => url($path . $fileName . '.jpg')], 200);


    }

    public function fetchFromUrl(Request $request)
    {

        $this->validate($request, [

            'urls' => 'required',
            'seller_id' => 'required|numeric',

        ]);
        //To be changed if the request will include the related product name
        $product_name = $request->product_name ? Str::slug($request->product_name) : 'product';

        $urls = json_decode($request->urls);

        $seller_id = $request->seller_id;

        $size = new stdClass();

        $size->width = $request->query->get('width');

        $size->height = $request->query->get('height');

        $result = [];
        foreach ($urls as $key => $url) {

            if ($key == 0 && !$this->validateURL($url))
                return response()->json(['message' => 'Main image is not valid'], 422);


            if ($this->validateURL($url)) {

                //     dump('start downloading  image '.date("h:i:sa"));

                $contents = file_get_contents($url);
                //   dump('finish downloading  image '.date("h:i:sa"));

                $info = pathinfo($url);

                $ext = $info['extension'];

                $file_name = $product_name . '_' . substr(Str::uuid()->toString(), 0, -18);

                $path = 'originals/' . $seller_id . '/';

                File::exists($path) ?:
                    File::makeDirectory($path, $mode = 0777, TRUE, TRUE);

                //         file_put_contents($path . $file_name . '.' . $ext, $contents);

                if (strtolower($ext) != 'jpg') {

                    $img = ImageIntervention::make($contents);

                    $img->encode('jpg')->save($path . $file_name . '.jpg');

                } else

                    file_put_contents($path . $file_name . '.' . $ext, $contents);

                $size->width ? $this->resizeImageCustomSize($size, $path, $seller_id, $file_name, 'jpg') :
                    $this->resizeImage($path, $seller_id, $file_name, 'jpg');


                array_push($result, url($path . $file_name . '.jpg'));
            }
        }

        return response()->json(['urls' => $result], 200);

    }

    function validateURL($url)
    {

        $headers = get_headers($url, 1);

        $sizeBytes = (int)$headers['Content-Length'];

        $sizeMb = $sizeBytes / (1024 * 1024);

        return stripos($headers[0], "200 OK") && $sizeMb <= 2 ? TRUE : FALSE;
    }

    public function resizeImageCustomSize($size, $original_path, $folder, $fileName, $ext = NULL)
    {

        $width = $size->width;

        $height = $size->height;

        $path = 'dims/' . $width . 'X' . ($height ? $height : $width) . '/' . $folder . '/';

        File::exists($path) ?:
            File::makeDirectory($path, $mode = 0777, TRUE, TRUE);

        $fileName = $fileName . ($ext ? ('.' . $ext) : '');

        $img = ImageIntervention::make($original_path . $fileName);

        $img->fit($width, $height, function ($constraint) {
            $constraint->upsize();
        })->encode('jpg')->save($path . '/' . $fileName);


    }

    public function resizeImage($original_path, $folder, $fileName, $ext)
    {

        foreach ($this->sizes as $size) {

            $width = $size['width'];

            $height = $size['height'];

            $path = 'dims/' . $width . 'X' . $height . '/' . $folder . '/';

            File::exists($path) ?:
                File::makeDirectory($path, $mode = 0777, TRUE, TRUE);

            $img = ImageIntervention::make($original_path . $fileName . '.' . $ext);

            $img->fit($width, $height, function ($constraint) {
                $constraint->upsize();
            })->encode('jpg')->save($path . '/' . $fileName . '.' . $ext);


        }
        //    dump('finish resize '.date("h:i:sa"));

    }


    public function getImage(Request $request)
    {

        $this->validate($request, [
            'url' => 'required',
        ]);

        $url = $request->url;


        $file_name = basename($url);


        $seller_id = basename(str_replace($file_name, '', $url));


        $width = array_key_exists('width', $_GET) ?
            $_GET['width'] : NULL;

        $height = array_key_exists('height', $_GET) ?
            $_GET['height'] : NULL;


        $original_path = 'originals/' . $seller_id . '/';

        if ($width) {

            $path = 'dims/' . $width . 'X' . ($height ? $height : $width) . '/' . $seller_id . '/' . $file_name . "?force_process=true";

            if (File::exists($original_path . $file_name)) {

                if (!File::exists($path)) {
                    if(array_search($width, array_column($this->available_sizes, 'width')) && array_search($height, array_column($this->available_sizes, 'height'))) {
                        
                        $size = new stdClass();

                        $size->width = $width;

                        $size->height = $height;

                        $this->resizeImageCustomSize($size, $original_path, $seller_id, $file_name);

                        return response()->json(['url' => url('/' . $path)], 200);
                    }else{
                        return asset('default.jpg');
                    }

                } else {

                    return response()->json(['url' => url('/' . $path)], 200);

                }


            }

            return response()->json(['msg' => 'Image not found'], 422);

        }


        return response()->json(['url' => url('/' . $original_path . $file_name)], 200);

    }


    public function delete(Request $request)
    {

        $urls = $request->urls ? json_decode($request->urls) : NULL;

        if (!$urls)
            return response()->json(['message' => 'urls not specified'], 422);


        foreach ($urls as $url) {

            $file_name = basename($url);
            $seller_id = basename(str_replace($file_name, '', $url));
            $original_path = 'originals/' . $seller_id . '/' . $file_name;

            $dimsFolder = glob('dims/*');

            foreach ($dimsFolder as $folder) {
                if (!File::isDirectory($folder))
                    continue;

                $subFolders = File::directories($folder);
                if (is_array($subFolders))
                    foreach ($subFolders as $subFolder) {
                        if (basename($subFolder) == $seller_id)
                            if (File::exists($subFolder . '/' . $file_name))
                                File::delete($subFolder . '/' . $file_name);
                    }

            }

            if (File::exists($original_path)) File::delete($original_path);

        }


        return response()->json(['message' => 'Successfully done!'], 200);

    }


}
