<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Intervention\Image\Facades\Image as ImageIntervention;
use stdClass;

class ImageController extends Controller
{

    protected $sizes =
        [

    ];
    protected $available_sizes =
        [

    ];

    protected $maximum_width = 1024;
    protected $maximum_height = 1024;

    public function handleUpload(Request $request)
    {
        ini_set('memory_limit', '1024M');

        $this->validate($request, [

            'image' => 'image|required|mimes:jpeg,png,jpg,gif,svg|max:3000',
            'seller_id' => 'required|numeric',

        ]);
        $seller_id = $request->seller_id;

        //To be changed if the request will include the related product name

        $product_name = $request->product_name ? Str::slug($request->product_name) : 'product';

        $file = $request->file('image');

        $fileName = $product_name . '-' . substr(Str::uuid()->toString(), 0, -13);

        $path = 'originals/' . $seller_id . '/';
        $size = new stdClass();

        $size->width = $request->query->get('width');

        $size->height = $request->query->get('height');

        File::exists($path) ?:
        File::makeDirectory($path, $mode = 0777, true, true);
        $url = $path . $fileName . '.jpg';

        if (strtolower($file->getClientOriginalExtension()) != 'jpg') {

            $img = ImageIntervention::make($file);
            // $img->resize($size->width,  $size->height , function ($constraint) {
            //     $constraint->aspectRatio();
            //     $constraint->upsize();
                
            // })->encode('jpg');
    
            $img->encode('jpg')->save($path . $fileName . '.jpg');

        } else {
            $file->move($path, $fileName . '.' . strtolower($file->getClientOriginalExtension()));
            $img = ImageIntervention::make($url);
        }


        // $size->width ? $this->resizeImageCustomSize($size, $path, $seller_id, $fileName, 'jpg') :
        // $this->resizeImage($path, $seller_id, $fileName, 'jpg');

        $this->checkImageDimensions($img, $path, $seller_id, $fileName, 'jpg');
        $this->checkImageSize($img, $url);

        return response()->json(['url' => url($path . $fileName . '.jpg')], 200);

    }

    public function fetchFromUrl(Request $request)
    {

        $this->validate($request, [

            'urls' => 'required',
            'seller_id' => 'required|numeric',

        ]);
        //To be changed if the request will include the related product name
        $product_name = $request->product_name ? Str::slug($request->product_name) : 'product';
        $urls = json_decode($request->urls);

        $seller_id = $request->seller_id;

        $size = new stdClass();

        $size->width = $request->query->get('width');

        $size->height = $request->query->get('height');

        $result = [];
        foreach ($urls as $key => $url) {

            if ($key == 0 && !$this->validateURL($url)) {
                return response()->json(['message' => 'Main image is not valid'], 422);
            }

            if ($this->validateURL($url)) {

                //     dump('start downloading  image '.date("h:i:sa"));

                $contents = file_get_contents($url);
                //   dump('finish downloading  image '.date("h:i:sa"));

                $info = pathinfo($url);

                $ext = $info['extension'];

                $file_name = $product_name . '-' . substr(Str::uuid()->toString(), 0, -18);

                $path = 'originals/' . $seller_id . '/';

                File::exists($path) ?:
                File::makeDirectory($path, $mode = 0777, true, true);

                //         file_put_contents($path . $file_name . '.' . $ext, $contents);

                if (strtolower($ext) != 'jpg') {

                    $img = ImageIntervention::make($contents);

                    $img->encode('jpg')->save($path . $file_name . '.jpg');

                } else {
                    file_put_contents($path . $file_name . '.' . $ext, $contents);
                }

                $size->width ? $this->resizeImageCustomSize($size, $path, $seller_id, $file_name, 'jpg') :
                $this->resizeImage($path, $seller_id, $file_name, 'jpg');

                array_push($result, url($path . $file_name . '.jpg'));
            }
        }

        return response()->json(['urls' => $result], 200);

    }

    public function copyFromUrl(Request $request)
    {
        ini_set('memory_limit', '1024M');
        $this->validate($request, [

            'urls' => 'required'

        ]);
        //To be changed if the request will include the related product name
        $product_name = $request->product_name ? Str::slug($request->product_name) : 'product';
        $urls = json_decode($request->urls);

        $size = new stdClass();
        $size->width = $request->query->get('width');
        $size->height = $request->query->get('height');
        $result = [];
        foreach ($urls as $key => $url) {
            if ($this->validateURL($url)) {
                $file_name = basename($url);
                $seller_id = basename(str_replace($file_name, '', $url));
                $original_path = 'originals/' . $seller_id . '/' . $file_name;
                $contents = file_get_contents($url);
                
                $info = pathinfo($url);
                $ext = $info['extension'];
                $file_name = str_replace(".".$ext,"",$file_name);
                $path = 'originals/' . $seller_id . '/';

                File::exists($path) ?:
                File::makeDirectory($path, $mode = 0777, true, true);
                $img = ImageIntervention::make($contents);
                if (strtolower($ext) != 'jpg') {
                    $img->encode('jpg');
                }
                $img->save($path . $file_name . '.jpg');
                $this->checkImageDimensions($img, $path, $seller_id, $file_name, 'jpg');
                $url = $path . $file_name . '.jpg';
                $this->checkImageSize($img, $url);
                array_push($result, url($url));
            }
        }

        return response()->json(['urls' => $result], 200);

    }

    public function checkImageDimensions($img, $path, $folder, $fileName, $ext){
        $full_path = $path . '/' . $fileName . '.' . $ext;
        list($width, $height) = getimagesize($full_path);
        if($width > $this->maximum_width || $height > $this->maximum_height){
            $result = $this->selectDimensions($width, $height);
            $width = $result['width'];
            $height = $result['height'];
            $img->fit((int)$width, (int)$height, function ($constraint) {
                $constraint->aspectRatio();
                $constraint->upsize();
            })
            ->save($path . '/' . $fileName . '.' . $ext);
        }
    }

    public function checkImageSize($img, $url){
        $size = File::size($url);
        if($size > 1048576 && $size < 2097152){
            $img->save($url, 30);
        }else if($size > 2097152 && $size < 3145728){
            $img->save($url, 20);
        }else if($size > 3145728){
            $img->save($url, 10);
        }
    }

    public function selectDimensions($width, $height){
        if($width > $this->maximum_width || $height > $this->maximum_height){
            $dimension = max([$width, $height]);
            $x = $this->maximum_width / $dimension;
            $width = (int) round($width * $x);
            $height = (int) round($height * $x);
            
        }
        return ['width'=>$width, 'height'=>$height];
    }

    public function validateURL($url)
    {

        $headers = get_headers($url, 1);
        if (!isset($headers['Content-Length'])) return false;
        $sizeBytes = (int) $headers['Content-Length'];

        $sizeMb = $sizeBytes / (1024 * 1024);

        return stripos($headers[0], "200 OK") && $sizeMb <= 3 ? true : false;
    }

    public function resizeImageCustomSize($size, $original_path, $folder, $fileName, $ext = null)
    {
        ini_set('memory_limit', '512M');
        $width = $size->width;

        $height = $size->height;

        $path = 'dims/' . $width . 'X' . ($height ? $height : $width) . '/' . $folder . '/';

        File::exists($path) ?:
        File::makeDirectory($path, $mode = 0777, true, true);

        $fileName = $fileName . ($ext ? ('.' . $ext) : '');

        //added by fesal
        $img = ImageIntervention::make($original_path . $fileName);
        // $width > $height ? $width=null : $height=null;

        $img->resize($width, $height, function ($constraint) {
            $constraint->aspectRatio();
            $constraint->upsize();
            
        })->encode('jpg');
        $img
        ->resizeCanvas(
            $width, // use the same width
            $height, // use the same height
            'center', // centre the image on the canves
            false, // don't apply relative mode 
            '#ffffff' // add a white background to the image
          );
      
        
          $img->save($path . '/' . $fileName);

    }

    public function resizeImage($original_path, $folder, $fileName, $ext)
    {
        ini_set('memory_limit', '1024M');

        foreach ($this->sizes as $size) {

            $width = $size['width'];

            $height = $size['height'];

            $path = 'dims/' . $width . 'X' . $height . '/' . $folder . '/';

            File::exists($path) ?:
            File::makeDirectory($path, $mode = 0777, true, true);

            //added by fesal
            $img = ImageIntervention::make($original_path . $fileName . '.' . $ext);
            // $width > $height ? $width=null : $height=null;
            $img->resize($width, $height, function ($constraint) {
                $constraint->aspectRatio();
                $constraint->upsize();
                
            })->encode('jpg');
            
            
            $img
            ->resizeCanvas(
                $width, // use the same width
                $height, // use the same height
                'center', // centre the image on the canves
                false, // don't apply relative mode 
                '#ffffff' // add a white background to the image
              );
          
            
              $img->save($path . '/' . $fileName . '.' . $ext);
            // $canvas

            // $img = ImageIntervention::make($original_path . $fileName . '.' . $ext);

            // $img->fit($width, $height, function ($constraint) {
            //     $constraint->aspectRatio();
            // })->encode('jpg')->resizeCanvas($width, $height)->save($path . '/' . $fileName . '.' . $ext);

        }
        //    dump('finish resize '.date("h:i:sa"));

    }

    public function getImage(Request $request)
    {

        $this->validate($request, [
            'url' => 'required',
        ]);

        $url = $request->url;

        $file_name = basename($url);

        $seller_id = basename(str_replace($file_name, '', $url));

        $width = array_key_exists('width', $_GET) ?
        $_GET['width'] : null;

        $height = array_key_exists('height', $_GET) ?
        $_GET['height'] : null;

        $original_path = 'originals/' . $seller_id . '/';

        if ($width) {

            $path = 'dims/' . $width . 'X' . ($height ? $height : $width) . '/' . $seller_id . '/' . $file_name . "?force_process=true";

            if (File::exists($original_path . $file_name)) {

                if (!File::exists($path)) {
                    if (array_search($width, array_column($this->available_sizes, 'width')) && array_search($height, array_column($this->available_sizes, 'height'))) {

                        $size = new stdClass();

                        $size->width = $width;

                        $size->height = $height;

                        $this->resizeImageCustomSize($size, $original_path, $seller_id, $file_name);

                        return response()->json(['url' => url('/' . $path)], 200);
                    } else {
                        return asset('default.jpg');
                    }

                } else {

                    return response()->json(['url' => url('/' . $path)], 200);

                }

            }

            return response()->json(['msg' => 'Image not found'], 422);

        }

        return response()->json(['url' => url('/' . $original_path . $file_name)], 200);

    }

    public function getImageWithNewSize(Request $request)
    {
        dd($request->folder);

        $this->validate($request, [
            'url' => 'required',
        ]);

        $url = $request->url;

        $file_name = basename($url);

        $seller_id = basename(str_replace($file_name, '', $url));

        $width = array_key_exists('width', $_GET) ?
        $_GET['width'] : null;

        $height = array_key_exists('height', $_GET) ?
        $_GET['height'] : null;

        $original_path = 'originals/' . $seller_id . '/';

        if ($width) {

            $path = 'dims/' . $width . 'X' . ($height ? $height : $width) . '/' . $seller_id . '/' . $file_name . "?force_process=true";

            if (File::exists($original_path . $file_name)) {

                if (!File::exists($path)) {
                    if (array_search($width, array_column($this->available_sizes, 'width')) && array_search($height, array_column($this->available_sizes, 'height'))) {

                        $size = new stdClass();

                        $size->width = $width;

                        $size->height = $height;

                        $this->resizeImageCustomSize($size, $original_path, $seller_id, $file_name);

                        return response()->json(['url' => url('/' . $path)], 200);
                    } else {
                        return asset('default.jpg');
                    }

                } else {

                    return response()->json(['url' => url('/' . $path)], 200);

                }

            }

            return response()->json(['msg' => 'Image not found'], 422);

        }

        return response()->json(['url' => url('/' . $original_path . $file_name)], 200);

    }

    public function delete(Request $request)
    {

        $urls = $request->urls ? json_decode($request->urls) : null;

        if (!$urls) {
            return response()->json(['message' => 'urls not specified'], 422);
        }

        foreach ($urls as $url) {

            $file_name = basename($url);
            $seller_id = basename(str_replace($file_name, '', $url));
            $original_path = 'originals/' . $seller_id . '/' . $file_name;

            $dimsFolder = glob('dims/*');

            foreach ($dimsFolder as $folder) {
                if (!File::isDirectory($folder)) {
                    continue;
                }

                $subFolders = File::directories($folder);
                if (is_array($subFolders)) {
                    foreach ($subFolders as $subFolder) {
                        if (basename($subFolder) == $seller_id) {
                            if (File::exists($subFolder . '/' . $file_name)) {
                                File::delete($subFolder . '/' . $file_name);
                            }
                        }

                    }
                }

            }

            if (File::exists($original_path)) {
                File::delete($original_path);
            }

        } 

        return response()->json(['message' => 'Successfully done!'], 200);

    }

    public function updateName(Request $request)
    {

        $urls = $request->urls ? json_decode($request->urls) : null;
        if (!$urls) {
            return response()->json(['message' => 'urls not specified'], 422);
        }

        foreach ($urls as $url) {
            $file_name = basename($url);
            if(substr($file_name,0,7) == 'product'){
                continue;
            }
            $seller_id = basename(str_replace($file_name, '', $url));
            $original_path = 'originals/' . $seller_id . '/' . $file_name;
            $new_original_path = 'originals/' . $seller_id . '/product-' . $file_name;
            if (File::exists($original_path)) {
                rename($original_path,$new_original_path);
            }
        }
        return response()->json(['message' => 'Successfully done!'], 200);


    }
    

}
